% DESCRIPTION
% -----------
% Calculates the conditional probability matrix for max 80 photons incident
% on a LOOPY with 8 bins (8 available modes)

% Creator: Andy Worsley
% Dept. of Physics, Univeristy of Oxford
% March 2008

% DEPENDENCIES
% ------------
% Requires "multinomial.m" to calculate multinomial coefficents
% Requires partition matrix 'Part' pages as calculated by "partition.m" 
%  - there MUST be a page for each photon number to be considered

% NOTES
% -----
% The loops labelled "1 BIN OCCUPIED"...."5 BINS OCCUPIED" can be commented
% out for calculating the column of the matrix for higher incident photon
% numbers (e.g. N>18) because these probabilities are close to zero. Doing
% this significantly improves the speed of the iteration for these higher
% photon numbers, which is otherwise long.

% The longest run I did was to calculate the convolution matrix for up to
% 30 incident photons. Above this, the probability of k=8 tends to unity.
% There may be no point calculating above about 35.


% ---- INITIALISE AND PREALLOCATE ----
i = 0; j = 0; k = 0; l = 0; m = 0; n = 0; o = 0; p = 0;
inc  = 0;
Prow = 0;
Pcol = 0;

clear P; clear M; clear D; clear C;

% USER VARIABLE NUMBER OF INCOMING PHOTONS & PROBABILITY OF MODES
% ---------------------------------------------------------------
N = 12; % Maximum number of incoming photons
P = [0.125,0.125,0.125,0.125,0.125,0.125,0.125,0.125]; % Probability for each mode
% ---------------------------------------------------------------

M = zeros(size(Part,1),1,N);
D = zeros(8,1);
Bcnt = ones(8,1);
c = zeros(8,1); % Accumulator for bin probabilities

C = zeros(9,81);

% ---- END OF INITIALISATION ----


% -----------
% ---START---
% -----------

% Trivial elements
C(1,1) = 1.0; C(2,2) = 1.0;

% ---- LOOP FOR NUMBER OF PHOTONS COMING IN ----
for inc = 1:N
    c = zeros(8,1); % Clear bin probabilities accumulator
    B = zeros(size(Part,1),9,8); % Temporary matrix which stores partitions in pages re: order of row
    Bcnt = ones(8,1);

    % ---- DATA PREPARATION ----
    % For each row, computes multinomial coefficent and counts the number of
    % entries of numbers 1 to 8 (degeneracy) and divides coefficient by
    % relevant factor
    for Prow = 1:max(find(Part(:,1,inc)))
        D = zeros(8,1);
        M(Prow,1,inc) = multinomial(inc, Part(Prow,:,inc));

        for Pcol = 1:8
            if ((Part(Prow,Pcol,inc) <= 8) && (Part(Prow,Pcol,inc) > 0))
                D(Part(Prow,Pcol,inc),1) = D(Part(Prow,Pcol,inc),1) + 1;
            end 
        end

        % Modifies multinomial coefficients
        for x = 1:8
            M(Prow,1,inc) = M(Prow,1,inc) / factorial(D(x,1));
        end

        % Finds the order by finding the index of
        % the furthest right non-zero element in Part

        % Puts rows into B on correct page corresponding to order
        B(Bcnt(max(find(Part(Prow,:,inc))),1), 1:8, max(find(Part(Prow,:,inc)))) =  Part(Prow,:,inc);
        % Puts coefficients in beside the rows
        B(Bcnt(max(find(Part(Prow,:,inc))),1), 9, max(find(Part(Prow,:,inc)))) =  M(Prow,1,inc);

        % Counts which row we are up to on each page
        Bcnt(max(find(Part(Prow,:,inc))),1) = Bcnt(max(find(Part(Prow,:,inc))),1) + 1;

    end

    % ---- DATA PREPARATION END ----


    for i = 1:8
    % 1 BIN OCCUPIED
        for x = 1:max(find(B(:,1,1)))
            c(1) = c(1) + B(x,9,1)*(P(i)^B(x,1,1));
        end

    if inc > 1
    for j = 1:8
    if (j~=i)           
    % 2 BINS OCCUPIED
        for x = 1:max(find(B(:,1,2)))
            c(2) = c(2) + B(x,9,2)*(P(i)^B(x,1,2))...
                *(P(j)^B(x,2,2));
        end


    if inc > 2
    for k = 1:8
    if ((k~=i)&&(k~=j))
    % 3 BINS OCCUPIED
        for x = 1:max(find(B(:,1,3)))
            c(3) = c(3) + B(x,9,3)*(P(i)^B(x,1,3))...
                *(P(j)^B(x,2,3))...
                *(P(k)^B(x,3,3));
        end

    if inc > 3
    for l = 1:8
    if ((l~=k)&&(l~=j)&&(l~=i))
    % 4 BINS OCCUPIED
        for x = 1:max(find(B(:,1,4)))
            c(4) = c(4) + B(x,9,4)*(P(i)^B(x,1,4))...
                *(P(j)^B(x,2,4))...
                *(P(k)^B(x,3,4))...
                *(P(l)^B(x,4,4));
        end

    if inc > 4
    for m = 1:8
    if ((m~=l)&&(m~=k)&&(m~=j)&&(m~=i))
    % 5 BINS OCCUPIED
        for x = 1:max(find(B(:,1,5)))
            c(5) = c(5) + B(x,9,5)*(P(i)^B(x,1,5))...
                *(P(j)^B(x,2,5))...
                *(P(k)^B(x,3,5))...
                *(P(l)^B(x,4,5))...
                *(P(m)^B(x,5,5));
        end

    if inc > 5
    for n = 1:8
    if ((n~=m)&&(n~=l)&&(n~=k)&&(n~=j)&&(n~=i))
    % 6 BINS OCCUPIED
        for x = 1:max(find(B(:,1,6)))
            c(6) = c(6) + B(x,9,6)*(P(i)^B(x,1,6))...
                *(P(j)^B(x,2,6))...
                *(P(k)^B(x,3,6))...
                *(P(l)^B(x,4,6))...
                *(P(m)^B(x,5,6))...
                *(P(n)^B(x,6,6));
        end

    if inc > 6
    for o = 1:8
    if ((o~=n)&&(o~=m)&&(o~=l)&&(o~=k)&&(o~=j)&&(o~=i))
    % 7 BINS OCCUPIED
        for x = 1:max(find(B(:,1,7)))
            c(7) = c(7) + B(x,9,7)*(P(i)^B(x,1,7))...
                *(P(j)^B(x,2,7))...
                *(P(k)^B(x,3,7))...
                *(P(l)^B(x,4,7))...
                *(P(m)^B(x,5,7))...
                *(P(n)^B(x,6,7))...
                *(P(o)^B(x,7,7));
        end

    if inc > 7
    for p = 1:8
    if ((p~=o)&&(p~=n)&&(p~=m)&&(p~=l)&&(p~=k)&&(p~=j)&&(p~=i))
    % 8 BINS OCCUPIED
        for x = 1:max(find(B(:,1,8)))
            c(8) = c(8) + B(x,9,8)*(P(i)^B(x,1,8))...
                *(P(j)^B(x,2,8))...
                *(P(k)^B(x,3,8))...
                *(P(l)^B(x,4,8))...
                *(P(m)^B(x,5,8))...
                *(P(n)^B(x,6,8))...
                *(P(o)^B(x,7,8))...
                *(P(p)^B(x,8,8));
        end

    end
    end
    end
    end
    end
    end
    end
    end
    end
    end
    end
    end
    end
    end
    end
    end
    end
    end
    end
    end
    end

    disp('i Completed'), disp(i);
    end

    % Fill Conditional Probability Matrix
    for x = 1:8
        C(x+1,inc+1) = c(x);
    end

    disp('Photon Number Completed'), disp(inc);
end % ---- INCOMING PHOTONS LOOP ----