% DESCRIPTION
% -----------
% Finds the multithermal state for two LOOPYs by testing a series of
% multithermal state parameters.

% Creator: Andy Worsley
% Dept. of Physics, Univeristy of Oxford
% March 2008

% NOTES
% -----
% I have not found a a least squares algorithm for an equation of the form
% P = Q[rho]R, where all the variables are matrices, so the first part of 
% the program reduces the problem to a standard p = Mx form for a least 
% squares fit, where the components of x are the diagonals of [rho].


% DEPENDENCIES
% ------------
% Function "Loss" must be available - creates loss matrices
% Function "nck" must be available - nchoosek with error suppressed for high
% coefficients
% P_kn1 and P_kn2 must be created, and click must be available


% INPUT
% -----
% P_kn1: convolution matrix for LOOPY 1
% P_kn2: convolution matrix for LOOPY 2
% P: joint statistics

% OUTPUT
% ------
% losses_bestrn: pair of loss parameters with the minimum fit residual
%      x_bestrn: column vector of the fitted input state
%           rho: diagonal matrix of fitted input state
%     parmspace: matrix of the residual at each mode/mean value, allowing a
%                2-D surface of parameter space to be drawn


% -----
% START
% -----

% ---- INITIALISE AND PREALLOCATE VARIABLES ----
N = 8; % Read highest considered photon pair number from convolution matrix size

n = 0; % Counting variables
m = 0;
prow = 0;
mrow = 0;
loopnum = 0;

mean = 0;
mode = 0;

% ------------------------------------------------------------------------
% USER DEFINED ITERATION PARAMETERS
% ------------------------------------------------------------------------
midmean = 2   ; % Puts centre of first iteration for mean parameter 1 at 0.5
midmode = 4   ; % Puts centre of first iteration for mode at 0.5
meanwidths = [2,   1,   0.5, 0.1 ]; %  , 1  , 0.5 , 0.1, 0.05 ]; % eg [0.5, ...] means first iteration will go +/- 0.5
modewidths = [3,   0.5, 0.1, 0.01 ]; % , 1  , 0.5 , 0.1, 0.05 ]; % eg [0.5, ...] means first iteration will go +/- 0.5

meanress =   [50,  20,  20,  20]; %, 50 , 20  , 20 , 10   ]; % eg [50, ...] means first iteration will take 50 points in the width
moderess =   [50,  50,  20,  10]; %, 50 , 20  , 20 , 10   ];
% ------------------------------------------------------------------------
% ------ USER DEFINED VALUES -----
%l1 = 0.8270;  % Loss parameter in 28-Feb-08 LOOPY 1 using Newton
%l2 = 0.8705;  % Loss parameter in 28-Feb-08 LOOPY 2 using Newton
%l1 = 0.8978; % Loss parameter in 5-Mar-08 LOOPY 1 using Newton
%l2 = 0.9172; % Loss parameter in 5-Mar-08 LOOPY 2 using Newton
l1 = 0.9112; % Loss parameter in 4-Apr-08 using search and magnify
l2 = 0.9264; % Loss parameter in 4-Apr-08 using search and magnify
% ------ USER DEFINED VALUES -----

L1 = Loss(l1,8); % Loss matrix in LOOPY 1
L2 = Loss(l2,8); % Loss matrix in LOOPY 2
M = zeros(9,9);
% Iteration parameter variables
meanwidth = 0;
modewidth = 0;
meanres   = 0;
moderes   = 0;

nummags   = size(moderess,2);
nummeans = max(meanress);
nummodes = max(moderess);

parmspace = zeros(nummeans,nummodes,nummags); % Space of possible loss combinations for each iteration

% Variables for providing user with information about how much of the
% program has been completed
countdone = 0;
percentdone = 0;
totalcount = sum(nummeans*nummodes);


% Create columns of vectors for storing test results
rho_mt = zeros(N+1,N+1,nummeans*nummodes);
resnorm = zeros(1,nummeans*nummodes);
meanmode = zeros(2,nummeans*nummodes);

% Columns for calculating optima
minresnorm = 0;
minrnind = 0;

% Return variables
rho_bestrn = zeros(N+1,N+1);
meanmode_bestrn = zeros(2,1);


% ---- END OF INITIALISATION ----


% Check P_kn1 and P_kn2 are the same size
if ((size(P_kn1,1)~=size(P_kn2,1)) + (size(P_kn1,2)~=size(P_kn2,2)))
    error('Convolution matrices must be the same size');
end


% ---- Magnification Loop ----
magcnt = 1;
while magcnt < nummags + 1
    
    meanwidth = meanwidths(magcnt);
    modewidth = modewidths(magcnt);
    nummeans = meanress(magcnt);
    nummodes = moderess(magcnt);
    
    % Create array of parameters
    if (midmean - meanwidth) < 0.0
        midmean = meanwidth;
    end
    if (midmode - modewidth) < 1.0
        midmode = modewidth + 1.0;
    end
    if (midmode + modewidth) > 1.0
        midmode = 5.0 - modewidth;
    end

    meanparms = linspace((midmean - meanwidth),(midmean + meanwidth), nummeans);
    modeparms = linspace((midmode - modewidth),(midmode + modewidth), nummodes);
    
    % ---- Mean Loop ----
    loopnum = 1;
    resnorm = zeros(1,nummeans*nummodes);
    for meancount = 1:(nummeans)
        mean = meanparms(meancount); % Assign mean
        
        % ---- Mode Loop ----
        for modecount = 1:(nummodes)
            mode = modeparms(modecount); % Assign number of modes
            
            % Find the residual
            rho_mt(1:9,1:9,loopnum) = multitherm(mean, mode, 8);
            M = L1*rho_mt(1:9,1:9,loopnum)*L2';
            
            resnorm(1,loopnum) = norm(P - M, 'fro');
            meanmode(1:2,loopnum) = [mean ; mode];
            
            % Create array for resnorm in paramater space - creates data
            % for a 2-D map of the residual norm for visual inspection
            parmspace(meancount,modecount,magcnt) = resnorm(1,loopnum);

            loopnum = loopnum + 1;
        end % ---- Mode Loop ----
        
 
    end % ---- Mean Loop ----

    % Find minimum residual norm, then return statistics and location
    [minresnorm, minrnind] = min(resnorm(1,:));
    meanmode_bestrn(:,1) = meanmode(:,minrnind);
    
    % Adjust parameters for next magnification level
    midmean = meanmode_bestrn(1,1);
    midmode = meanmode_bestrn(2,1);
    
    magcnt
    magcnt = magcnt + 1;
end
% ---- End of Magnification Loop ----


% ---- Final Return of Data ----
meanmode_bestrn(:,1) = meanmode(:,minrnind);
rho_bestrn(1:9,1:9) = rho_mt(1:9,1:9,minrnind);