%  FITTOMULTITHERM
% Takes a state rho and then varies s in a multithermal state M with
% the same mean in order to minimise norm(M - rho).

% Creator: Andy Worsley
% Dept. of Physics, Univeristy of Oxford
% March 2008

% Variation is done using Newton's method. The parameters h and gamma
% usually have to be adjusted for each specific problem to give the most
% efficient optimisation.

% DEPENDENCIES
% Function meanvar.m must be available - calculates the mean of the state

% INPUT: rho - the state to which M is fitted
% OUTPUTS: s, mean, resnorm


% Max number of photons to consider i.e. size of state
N = size(rho,2) - 1; % Read highest considered photon pair number from rho size

% ---- USER DEFINED PARAMETERS ----
h = 0.1; % Distance in parameter space over which to numerically calculate derivatives
gamma = 50; % Multiplication factor for direction vector
s_0 = 2; % An initial guess is provided e.g. 1 mode.
Imax = 500; % Maximum number of iterations
% ---------------------------------

iter = 1;
d = zeros(1,Imax);

s = zeros(1,Imax);
s(1) = s_0;

T = zeros(2,3);
D = [0,0];

% ---------
%   START
% ---------

mean = meanvar(rho);

for iter = 1:Imax
    
    % ---- Calculating the parameter points for derivatives ----
    T(1,1) = s(1,iter); % Load point
    T(1,2) = s(1,iter) + h; % Load point + delta
    T(1,3) = s(1,iter) - h; % Load point - delta
    
    % ---- Calculating the function values ----
    % The function to be minimised is the residual of the fit
    for i=1:3
        M = multitherm(mean(1), T(1,i), N);
        T(2,i) = norm(M - rho, 'fro');
    end
    
    % ---- Derivatives ----
    % 1st derivative
    D(1) = (T(2,2) - T(2,3))/(2*h);
    % 2nd derivative
    D(2) = (T(2,2) + T(2,3) - T(2,1))/(h^2);
    
    % ---- Moving to a new point in parameter space ----
    % Points are recorded in 3 x Imax matrix, s, where Imax = max number of iterations
    if iter < Imax
       d(1,iter) = D(1)/D(2);
       s(1,iter+1) = s(1,iter) - gamma*d(1,iter);
    end
    
end
 